#ifndef BNR_H
#define BNR_H

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef HAVE_MPICHBNR_API

#ifdef MPICHBNR_EXPORTS
#define MPICHBNR_API __declspec(dllexport)
#else
#define MPICHBNR_API __declspec(dllimport)
#endif

#else
#define MPICH_BNR_API
#endif

struct BNR_Info_struct {
    int cookie;
    char *key, *value;
    struct BNR_Info_struct *next;
};

typedef struct BNR_Info_struct *BNR_Info;
typedef int BNR_gid;

#define BNR_INVALID_GROUP	((BNR_gid)-1)
#define BNR_GROUP_NULL		((BNR_gid)0)
#define BNR_SUCCESS			0
#define BNR_FAIL			-1
#define BNR_MAXATTRLEN		64
#define BNR_MAXVALLEN		3*1024

/*******************************************************
 * Construction / Destruction of the BNR interface
 *
 */

/* Initializes the bnr interface */
MPICHBNR_API int BNR_Init();

/* frees any internal resources
 * No BNR calls may be made after BNR_Finalize
 */
MPICHBNR_API int BNR_Finalize();


/*******************************************************
 * Group management functions
 *
 */

/* returns primary group id assigned at creation */
MPICHBNR_API int BNR_Group( BNR_gid *mygid );

/* returns BNR_GROUP_NULL if no parent */
MPICHBNR_API int BNR_Parent( BNR_gid *parent_gid );

/* returns rank in group */
MPICHBNR_API int BNR_Rank( BNR_gid group, int *myrank );

/* Allocates a new, unique group id which may be used 
 * in multiple spawn calls until it is closed.
 * Collective over the local group.
 * Cannot be fenced until after it has been closed.
 */
MPICHBNR_API int BNR_Open_gid( BNR_gid local_group, BNR_gid *new_gid );

/* Closes an open group.
 * Collective over the group that opened it.
 */
MPICHBNR_API int BNR_Close_gid( BNR_gid gid );

/* frees gid for re-use. */
MPICHBNR_API int BNR_Free_gid( BNR_gid gid );

/* Calling process must be in the local group and
 * must not be in the remote group.  Collective 
 * over the union of the two groups. 
 */
MPICHBNR_API int BNR_Merge( 
    BNR_gid local_group, BNR_gid remote_group, BNR_gid *new_group );


/*******************************************************
 * Process management functions
 *
 */

/* not collective.
 * remote_group is an open BNR_gid and may be passed to Spawn 
 * multiple times. It is not valid until it is closed.  
 * BNR_Spawn will fail if remote_group is closed or uninitialized.
 * notify_fn is called if a process exits, and gets the
 * group, rank, and return code. argv and env
 * arrays are null terminated.  The caller's group is the
 * parent of the spawned processes.
 */

MPICHBNR_API int BNR_Spawn( 
    BNR_gid remote_group, 
    int count, char *command, char *argv[], char *env[], 
    BNR_Info info, int array_of_errcodes[], int (notify_fn)(BNR_gid group, 
    int rank, int exit_code) );

/* kills processes in group given by gid.  This
 * can be used, for example, during spawn_multiple
 * when a spawn fails, to kill off groups already
 * spawned before returning failure 
 */
MPICHBNR_API int BNR_Kill( BNR_gid gid );


/*******************************************************
 * Attribute management functions
 *
 */

/* puts attr-value pair for retrieval by other
 * processes in group;  attr is a string of
 * length < BNR_MAXATTRLEN, val is string of
 * length < BNR_MAXVALLEN 
 * rank_advice tells BNR where the Get is likely to be called from.
 * rank_advice can be -1 for no advice.
 */
MPICHBNR_API int BNR_Put( 
    BNR_gid group, char *attr, char *val, int rank_advice );

/* matches attr, retrieves corresponding value
 * into val, which is a buffer of
 * length = BNR_MAXVALLEN 
 */
MPICHBNR_API int BNR_Get( BNR_gid group, char *attr, char *val );

/* barriers all processes in group; puts done
 * before the fence are accessible by gets after
 * the fence 
 */
MPICHBNR_API int BNR_Fence( BNR_gid );	   


/*******************************************************
 * Global asynchronous put/get functions
 *
 * The following are needed for publishing.  They require no fence, since they
 * are not assumed to be either scalable or local.  The inevitable race 
 * condition is just accepted.
 */ 

/* deposits attr-value pair for access */
MPICHBNR_API int BNR_Deposit( char *attr, char *value );

/* withdraws attr-value pair */
MPICHBNR_API int BNR_Withdraw( char *attr, char *value );

/* finds value of attribute */
MPICHBNR_API int BNR_Lookup( char *attr, char *value );


#if defined(__cplusplus)
}
#endif 

#endif
